/*
 * Copyright (C) 2016 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */

#include <ubuntu-app-launch/registry.h>

using namespace ubuntu::app_launch;
namespace ual = ubuntu::app_launch;

#include <gtest/gtest.h>
#include <gmock/gmock.h>

using namespace ::testing;

namespace
{

    class MockUALRegistry : public ual::Registry {
    public:
        MockUALRegistry()
        {
        }

        MOCK_METHOD1(installedApps, std::list<std::shared_ptr<ual::Application>>(std::shared_ptr<ual::Registry>));
    };

    class MockUALApplication : public ual::Application {
    public:
        class MockInfo;
        class MockInstance;

        MockUALApplication(const ual::AppID& app_id,
                           const std::shared_ptr<MockInfo>& info = {})
            : _app_id(app_id),
              _info(info)
        {
        }

        ual::AppID appId()
        {
            return _app_id;
        }

        std::shared_ptr<ual::Application::Info> info()
        {
            return _info;
        }

        static std::shared_ptr<MockUALApplication> create(ual::AppID app_id,
                                                          std::shared_ptr<MockInfo> info = {})
        {
            std::shared_ptr<MockUALApplication> result{new MockUALApplication(app_id, info)};
            return result;
        }

        class MockInfo : public ual::Application::Info
        {
        public:
            MockInfo(const std::string& title,
                     const std::string& description,
                     const std::string& iconPath)
                : _name(ual::Application::Info::Name::from_raw(title)),
                _description(ual::Application::Info::Description::from_raw(description)),
                _iconPath(ual::Application::Info::IconPath::from_raw(iconPath)),
                _defaultDept(ual::Application::Info::DefaultDepartment::from_raw("")),
                _ssPath(ual::Application::Info::IconPath::from_raw("")),
                _keywords(ual::Application::Info::Keywords::from_raw(std::vector<std::string>{}))
            {
                DefaultValue<const ual::Application::Info::DefaultDepartment&>::Set(_defaultDept);
                DefaultValue<const ual::Application::Info::IconPath&>::Set(_ssPath);
                DefaultValue<const ual::Application::Info::Keywords&>::Set(_keywords);
            }

            const ual::Application::Info::Name& name()
            {
                return _name;
            }

            const ual::Application::Info::Description& description()
            {
                return _description;
            }

            const ual::Application::Info::IconPath& iconPath()
            {
                return _iconPath;
            }

            MOCK_METHOD0(defaultDepartment, const ual::Application::Info::DefaultDepartment&());
            MOCK_METHOD0(screenshotPath, const ual::Application::Info::IconPath&());
            MOCK_METHOD0(keywords, const ual::Application::Info::Keywords&());

            MOCK_METHOD0(splash, ual::Application::Info::Splash());
            MOCK_METHOD0(supportedOrientations, ual::Application::Info::Orientations());
            MOCK_METHOD0(rotatesWindowContents, ual::Application::Info::RotatesWindow());
            MOCK_METHOD0(supportsUbuntuLifecycle, ual::Application::Info::UbuntuLifecycle());

        private:
            ual::Application::Info::Name _name;
            ual::Application::Info::Description _description;
            ual::Application::Info::IconPath _iconPath;
            ual::Application::Info::DefaultDepartment _defaultDept;
            ual::Application::Info::IconPath _ssPath;
            ual::Application::Info::Keywords _keywords;
        };

        class MockInstance : public ual::Application::Instance
        {
        public:
            MOCK_METHOD0(isRunning, bool());
            MOCK_METHOD0(logPath, std::string());
            MOCK_METHOD0(primaryPid, pid_t());
            MOCK_METHOD1(hasPid, bool(pid_t));
            MOCK_METHOD0(pids, std::vector<pid_t>());
            MOCK_METHOD0(pause, void());
            MOCK_METHOD0(resume, void());
            MOCK_METHOD0(stop, void());
        };

        MOCK_METHOD0(hasInstances, bool());
        MOCK_METHOD0(instances, std::vector<std::shared_ptr<ual::Application::Instance>>());
        MOCK_METHOD1(launch, std::shared_ptr<ual::Application::Instance>(const std::vector<ual::Application::URL>&));
        MOCK_METHOD1(launchTest, std::shared_ptr<ual::Application::Instance>(const std::vector<ual::Application::URL>&));

    private:
        ual::AppID _app_id;
        std::shared_ptr<MockInfo> _info;
    };

} // namespace

